"""Illustration of the autocorrelation issue."""

import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
import numpy as np
from scipy.stats import gaussian_kde

# Two independent variables
rng = np.random.default_rng(0)
x_orig = rng.normal(size=5)
y_orig = rng.normal(size=5)


# Fit polynomials to the points
t = np.arange(4*24 + 1) / 24

xc = np.polyfit(np.arange(5), x_orig, deg=4)
x_ac = xc[4] + xc[3] * t + xc[2] * t**2 + xc[1] * t**3 + xc[0] * t**4

yc = np.polyfit(np.arange(5), y_orig, deg=4)
y_ac = yc[4] + yc[3] * t + yc[2] * t**2 + yc[1] * t**3 + yc[0] * t**4


# Estimate the kernel with KDE, default bandwidth selected by sample size
marginal = np.linspace(-2.5, 2.5, 50)


# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)
plt.rc("xtick", bottom=False, labelbottom=False)
plt.rc("ytick", left=False, labelleft=False)

fig = plt.figure(figsize=(8, 3))
gs = GridSpec(1, 5, width_ratios=[1, 3, 1, 3, 1], wspace=0.05)

left_main = fig.add_subplot(gs[0,1])
left_main.scatter(x_orig, y_orig, marker=".", color="k")

left_hist = fig.add_subplot(gs[0,0], sharey=left_main)
left_hist.plot(gaussian_kde(y_orig)(marginal), marginal, color="C0")
left_hist.invert_xaxis()
left_hist.set_title("(a)", loc="left")

right_main = fig.add_subplot(gs[0,3])
right_main.scatter(x_ac, y_ac, marker=".", color="gray")
right_main.scatter(x_orig, y_orig, marker=".", color="k")
right_main.set_title("(b)", loc="left")

right_hist = fig.add_subplot(gs[0,4], sharey=right_main)
right_hist.plot(gaussian_kde(y_ac)(marginal), marginal, color="C0")

plt.savefig("autocorrelation_example.pdf", transparent=True)

