"""Correlation coefficients as a function of outlier delta/proportion.

This script is quite slow because it does not implement any parallelization.
Built-in parallelization of ennemi cannot be used because the estimates are
required to be completely independent."""

from ennemi import estimate_corr
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as stats

SAMPLES = 20
ITERATIONS = 100

rng = np.random.default_rng(2020_06_02)

# Increasing outlier delta
pearson_d = np.empty((ITERATIONS, SAMPLES))
spearman_d = np.empty((ITERATIONS, SAMPLES))
mi_d = np.empty((ITERATIONS, SAMPLES))
deltas = np.linspace(0, 20, SAMPLES)

# Increasing proportion of outliers
pearson_p = np.empty((ITERATIONS, SAMPLES))
spearman_p = np.empty((ITERATIONS, SAMPLES))
mi_p = np.empty((ITERATIONS, SAMPLES))
props = np.linspace(0, 0.2, SAMPLES)

for i in range(ITERATIONS):
    if i % 10 == 0:
        print(f"Iteration {i+1}/{ITERATIONS}")

    cov = np.asarray([[1, 0.6], [0.6, 1]])
    data = rng.multivariate_normal([0, 0], cov, 500)
    y = data[:,1]

    # Delta
    for (j, delta) in enumerate(deltas):
        x = data[:,0].copy()
        x[0] += delta

        pearson_d[i, j] = np.corrcoef(x, y)[0,1]
        spearman_d[i, j] = stats.spearmanr(x, y)[0]
        mi_d[i, j] = estimate_corr(y, x)
        
    # Proportion
    for (j, p) in enumerate(props):
        x = data[:,0].copy()
        x[:int(p*len(x))] = 10

        pearson_p[i, j] = np.corrcoef(x, y)[0,1]
        spearman_p[i, j] = stats.spearmanr(x, y)[0]
        mi_p[i, j] = estimate_corr(y, x)
        

# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)
plt.rc("xtick", labelsize="small")
plt.rc("ytick", labelsize="small")

fig, (ax1, ax2) = plt.subplots(ncols=2, figsize=(8, 4), sharey=True)

# Left: delta
ax1.plot(deltas, np.quantile(mi_d, 0.5, axis=0), label="MI", ls="-")
ax1.fill_between(deltas,
    np.quantile(mi_d, 0.25, axis=0),
    np.quantile(mi_d, 0.75, axis=0), alpha=0.2)

ax1.plot(deltas, np.quantile(spearman_d, 0.5, axis=0), label="Spearman", ls="-.")
ax1.fill_between(deltas,
    np.quantile(spearman_d, 0.25, axis=0),
    np.quantile(spearman_d, 0.75, axis=0), alpha=0.2)

ax1.plot(deltas, np.quantile(pearson_d, 0.5, axis=0), label="Pearson", ls="--")
ax1.fill_between(deltas,
    np.quantile(pearson_d, 0.25, axis=0),
    np.quantile(pearson_d, 0.75, axis=0), alpha=0.2)

ax1.legend(loc="lower left")
ax1.set_xticks([0, 5, 10, 15, 20])
ax1.set_xlabel("$x$ delta of the outlier point")
ax1.set_ylabel("Correlation coefficient")
ax1.set_title("(a)", loc="left")
ax1.grid()

# Right: proportion
ax2.plot(props, np.quantile(mi_p, 0.5, axis=0), label="MI", ls="-")
ax2.fill_between(props,
    np.quantile(mi_p, 0.25, axis=0),
    np.quantile(mi_p, 0.75, axis=0), alpha=0.2)

ax2.plot(props, np.quantile(spearman_p, 0.5, axis=0), label="Spearman", ls="-.")
ax2.fill_between(props,
    np.quantile(spearman_p, 0.25, axis=0),
    np.quantile(spearman_p, 0.75, axis=0), alpha=0.2)

ax2.plot(props, np.quantile(pearson_p, 0.5, axis=0), label="Pearson", ls="--")
ax2.fill_between(props,
    np.quantile(pearson_p, 0.25, axis=0),
    np.quantile(pearson_p, 0.75, axis=0), alpha=0.2)

ax2.legend(loc="lower left")
ax2.set_xlabel("Proportion of outliers")
ax2.set_title("(b)", loc="left")
ax2.set_xticks([0, 0.05, 0.1, 0.15, 0.2])
ax2.set_xticklabels(["0%", "5%", "10%", "15%", "20%"])
ax2.grid()

plt.savefig("outlier_ci.pdf", transparent=True)

