"""Precision of MI correlation as a function of sample size.

This script is quite slow because it does not implement any parallelization.
Built-in parallelization of ennemi cannot be used because the estimates are
required to be completely independent."""

from ennemi import estimate_corr
import matplotlib.pyplot as plt
import numpy as np

SAMPLES = 20
ITERATIONS = 150

rng = np.random.default_rng(2020_06_01)
mi_norm = np.empty((ITERATIONS, SAMPLES))
mi_low = np.empty((ITERATIONS, SAMPLES))
sample_sizes = np.logspace(np.log10(50), 4, SAMPLES, dtype=np.int32)

for i in range(ITERATIONS):
    if i % 10 == 0:
        print(f"Iteration {i+1}/{ITERATIONS}")

    cov1 = np.asarray([[1, 0.6], [0.6, 1]])
    cov2 = np.asarray([[1, 0.2], [0.2, 1]])
    norm_data = rng.multivariate_normal([0, 0], cov1, 10**4)
    low_data = rng.multivariate_normal([0, 0], cov2, 10**4)
    for (j, ss) in enumerate(sample_sizes):
        mi_norm[i, j] = estimate_corr(norm_data[:ss,0], norm_data[:ss,1])
        mi_low[i, j] = estimate_corr(low_data[:ss,0], low_data[:ss,1])

# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)
plt.rc("xtick", labelsize="small")
plt.rc("ytick", labelsize="small")

fig, ax = plt.subplots(figsize=(4, 4))
ax.axhline(0.6, linestyle="--", color="k")
ax.axhline(0.2, linestyle="--", color="k")

ax.plot(sample_sizes, np.quantile(mi_norm, 0.5, axis=0), label="$\\rho=0.6$", color="C3")
ax.fill_between(sample_sizes,
    np.quantile(mi_norm, 0.25, axis=0),
    np.quantile(mi_norm, 0.75, axis=0), color="C3", alpha=0.2)
ax.plot(sample_sizes, np.quantile(mi_low, 0.5, axis=0), label="$\\rho=0.2$", color="C4")
ax.fill_between(sample_sizes,
    np.quantile(mi_low, 0.25, axis=0),
    np.quantile(mi_low, 0.75, axis=0), color="C4", alpha=0.2)

ax.legend(loc="lower right")
ax.set_xscale("log")
ax.set_xlabel("Sample size")
ax.set_ylabel("Correlation coefficient")
ax.grid()

plt.savefig("sample_size_ci.pdf", transparent=True)

