"""Estimate MI correlation of CS and other variables with lag."""

from ennemi import estimate_corr
from helpers import estimate_and_print_stats, get_estimate_mi_callback
import numpy as np
import pandas as pd

# Load the data and masks
data = pd.read_hdf("scaled-data.hdf5", "scaled_data")

# Lag up to 24 hours in 1-hour steps
lags = np.arange(0, 24+1)

# Estimate MI on random samples of the data
# Take the average of a few samples to improve the accuracy
INTEREST = "CS"
REPEATS = 3
FRAC = 0.05
rng = np.random.default_rng(2022_03_25)


def estimate_unconditional(lags):
    columns = [c for c in data.columns if c != INTEREST]
    callback = get_estimate_mi_callback(data[columns], lags, REPEATS)

    result = pd.DataFrame(data=None, columns=columns)
    for _ in range(REPEATS):
        # CAREFUL! If we directly sample a subset of the data, we cannot use lags
        # (because the data points are no longer adjacent in time).
        # Instead, we create a mask that selects 5% of the time points.
        # This mask selects the observations of the variable of interest,
        # and the corresponding covariate observations are picked 0, ..., 24 hours back in time.
        mask = rng.choice([False, True], data.shape[0], replace=True, p=[1-FRAC, FRAC])

        print(f"-- {np.sum(mask)} observations")
        subresult = estimate_corr(data[INTEREST], data[columns], drop_nan=True,
            mask=mask, lag=lags, callback=callback)
            
        result = subresult.add(result, fill_value=0)
    
    return result / REPEATS

def estimate_conditional_doy_hour(lags):
    columns = [c for c in data.columns if c != INTEREST]
    callback = get_estimate_mi_callback(data[columns], lags, REPEATS)

    result = pd.DataFrame(data=None, columns=columns)
    for _ in range(REPEATS):
        mask = rng.choice([False, True], data.shape[0], replace=True, p=[1-FRAC, FRAC])

        print(f"-- {np.sum(mask)} observations")
        subresult = estimate_corr(data[INTEREST], data[columns], drop_nan=True,
            mask=mask, lag=lags, callback=callback, cond=data[["DayOfYear", "Hour"]])
            
        result = subresult.add(result, fill_value=0)
    
    return result / REPEATS

estimate_and_print_stats(estimate_unconditional, lags).to_csv(
    "lag-CS-all-RESULTS.csv")
estimate_and_print_stats(estimate_conditional_doy_hour, lags).to_csv(
    "lag-CS-all-doy-hour-RESULTS.csv")
