"""Estimate the pairwise correlation matrix."""

import numpy as np
import pandas as pd
from ennemi import pairwise_corr
from helpers import get_pairwise_mi_callback
from datetime import datetime, timezone

# Load the preprocessed data
data = pd.read_hdf("scaled-data.hdf5", "scaled_data")

# Estimate pairwise MI on a random sample of the data
# We take the average of a few such samples; this improves accuracy slightly
REPEATS = 3
rng = np.random.default_rng(2022_03_25)

for (cond, frac, suffix) in [(None, 0.05, "-5%-none"),
                             (["DayOfYear"], 0.05, "-5%-doy"),
                             (["DayOfYear", "Hour"], 0.05, "-5%-doy-hour")]:
    result = pd.DataFrame(data=None, columns=data.columns)

    print(f"Analyzing with condition \"{suffix}\"")
    callback = get_pairwise_mi_callback(data, REPEATS)
    start_time = datetime.now(timezone.utc)

    for _ in range(REPEATS):
        # For pairwise_corr, we could also use data.sample(...),
        # but we use masking to be consistent with the correct usage of estimate_corr.
        # data.sample() destroys the time series structure while masking preserves it;
        # this is important if estimate_corr is used with lags
        mask = rng.choice([False, True], data.shape[0], replace=True, p=[1-frac, frac])
        print(f"-- {np.sum(mask)} observations")

        if cond is not None: cond_data = data[cond]
        else: cond_data = None

        subresult = pairwise_corr(data, cond=cond_data,
            callback=callback, drop_nan=True, mask=mask)
        result = subresult.add(result, fill_value=0)

    end_time = datetime.now(timezone.utc)
    result /= REPEATS
    result.to_csv(f"pairwise{suffix}-RESULTS.csv")

    total_time = end_time - start_time
    print("Total time         :", total_time)
    print()
