"""Methods used across multiple analysis scripts."""

from datetime import datetime, timezone

def estimate_and_print_stats(estimation_method, lags, *args):
    """Run the estimation method and print timing information."""
    print(f"-- {estimation_method.__name__}")

    start_time = datetime.now(timezone.utc)
    result = estimation_method(lags, *args)
    end_time = datetime.now(timezone.utc)
    total_time = end_time - start_time

    total_vars = len(result.columns)
    print()
    print("Lags               :", len(lags))
    print("Variables          :", total_vars)
    print("Total time         :", total_time)
    print("Time per variable  :", total_time / total_vars)
    print("Time per estimation:", total_time / (total_vars * len(lags)))
    print()

    return result


def create_time_mask(data, hour, minute):
    """Return a boolean Series of the indices matching the given time."""
    return (data.index.hour == hour) & (data.index.minute == minute)


def get_estimate_mi_callback(x, lags, factor=1):
    """Return a callback that prints a line for every 10% of estimation."""
    total_tasks = x.shape[1] * len(lags) * factor
    tasks_done = 0

    def callback(var_index, lag):
        nonlocal tasks_done
        tasks_done += 1

        cur_percent = (tasks_done / total_tasks) * 100
        prev_percent = ((tasks_done - 1) / total_tasks) * 100

        if cur_percent // 10 > prev_percent // 10:
            print(f"{datetime.now():%X}: {int(cur_percent):>3}% ({tasks_done}/{total_tasks})")

    return callback

def get_pairwise_mi_callback(x, factor=1):
    """Return a callback that prints a line for every 10% of estimation."""
    total_tasks = (x.shape[1] * (x.shape[1] - 1) // 2) * factor
    tasks_done = 0

    def callback(var_index, lag):
        nonlocal tasks_done
        tasks_done += 1

        cur_percent = (tasks_done / total_tasks) * 100
        prev_percent = ((tasks_done - 1) / total_tasks) * 100

        if cur_percent // 10 > prev_percent // 10:
            print(f"{datetime.now():%X}: {int(cur_percent):>3}% ({tasks_done}/{total_tasks})")

    return callback
