"""Temperature autocorrelation at Helsinki Kaisaniemi."""

from ennemi import estimate_corr
import matplotlib.pyplot as plt
from matplotlib.cm import get_cmap
import numpy as np
import pandas as pd
from scipy.optimize import curve_fit

# Load the data
# Temperature has a roughly symmetric distribution, no need to transform
full_data = pd.read_hdf("kaisaniemi.hdf5", "raw_data")



# Calculate unconditional MI
mask = (full_data.index.hour == 13) # 15:00 Helsinki time
lags = -np.arange(1, 24*7)

print(f"Observations: {np.sum(mask)}")
auto_mi = estimate_corr(full_data["Temperature"], full_data["Temperature"],
    mask=mask, lag=lags)

# Calculate MI conditional on DOY
auto_mi_doy = estimate_corr(full_data["Temperature"], full_data["Temperature"],
    mask=mask, lag=lags, cond=full_data["DayOfYear"])



# Calculate Pearson correlation
def pearson_autocorr(x):
    result = np.empty(lags.shape)

    # Calculate Pearson correlation with lagged variable
    # This code is copied from ennemi._driver._lagged_mi()
    for i in range(len(lags)):
        lag = lags[i]
        min_lag = np.min(lags)
        max_lag = 0
        xs = x[max_lag-lag : len(x)-lag+min(min_lag, 0)]
        ys = x[max_lag : len(x)+min(min_lag, 0)]

        mask_subset = mask[max_lag : len(x)+min(min_lag, 0)]
        xs = xs[mask_subset]
        ys = ys[mask_subset]

        result[i] = np.corrcoef(xs, ys)[0,1]

    return result

auto_pearson = pearson_autocorr(full_data["Temperature"])

# Fit a trigonometric model and calculate partial correlation
YEAR = 2 * np.pi/365
HALF_YEAR = 2 * YEAR

def model(x, a1, a2, b1, b2, c):
    return a1*np.sin(x*YEAR) + a2*np.cos(x*YEAR) + \
        b1*np.sin(x*HALF_YEAR) + b2*np.cos(x*HALF_YEAR) + c

popt, _ = curve_fit(model, full_data["DayOfYear"], full_data["Temperature"])
pred = model(full_data["DayOfYear"], *popt)

residuals = full_data["Temperature"] - pred
auto_pearson_doy = pearson_autocorr(residuals)



# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)
plt.rc("xtick", labelsize="small")
plt.rc("ytick", labelsize="small")

fig, ax = plt.subplots(figsize=(8, 4))
c = get_cmap("tab20").colors

ax.plot(auto_mi, label="MI, unconditional", color=c[8])
ax.plot(lags, auto_pearson, "--", label="Pearson, unconditional", color=c[9])

ax.plot(auto_mi_doy, label="MI, conditional on DOY", color=c[6])
ax.plot(lags, auto_pearson_doy, "--", label="Partial correlation, given DOY", color=c[7])

ax.invert_xaxis()
ax.set_xticks(-24 * np.arange(0, 8))
ax.set_xticklabels([f"+{i} days" for i in np.arange(0, 8)])
ax.grid()
ax.legend()
ax.set_ylabel("Correlation coefficient")

plt.savefig("temperature.pdf", transparent=True)
